# coding: utf-8

"""
Wallee AG Python SDK

This library allows to interact with the Wallee AG payment service.

Copyright owner: Wallee AG
Website: https://en.wallee.com
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from datetime import datetime
from pydantic import Field, StrictInt, StrictStr
from typing import List, Optional
from typing_extensions import Annotated
from wallee.models.metric_usage_list_response import MetricUsageListResponse

from wallee.api_client import ApiClient, RequestSerialized
from wallee.api_response import ApiResponse
from wallee.rest import RESTResponseType



class ConsumedResourcesService:
    def __init__(self, configuration):
        self.api_client = ApiClient(configuration=configuration)


    @validate_call
    def get_spaces_consumed_resources(
        self,
        start_date: Annotated[datetime, Field(description="The start date of the period for which the consumed resources are to be returned.")],
        end_date: Annotated[datetime, Field(description="The end date of the period for which the consumed resources are to be returned.")],
        space: Annotated[StrictInt, Field(description="Specifies the ID of the space the operation should be executed in.")],
        expand: Optional[List[StrictStr]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> MetricUsageListResponse:
        """List consumed resources



        :param start_date: The start date of the period for which the consumed resources are to be returned. (required)
        :type start_date: datetime
        :param end_date: The end date of the period for which the consumed resources are to be returned. (required)
        :type end_date: datetime
        :param space: Specifies the ID of the space the operation should be executed in. (required)
        :type space: int
        :param expand:
        :type expand: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_spaces_consumed_resources_serialize(
            start_date=start_date,
            end_date=end_date,
            space=space,
            expand=expand,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "MetricUsageListResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_spaces_consumed_resources_with_http_info(
        self,
        start_date: Annotated[datetime, Field(description="The start date of the period for which the consumed resources are to be returned.")],
        end_date: Annotated[datetime, Field(description="The end date of the period for which the consumed resources are to be returned.")],
        space: Annotated[StrictInt, Field(description="Specifies the ID of the space the operation should be executed in.")],
        expand: Optional[List[StrictStr]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[MetricUsageListResponse]:
        """List consumed resources



        :param start_date: The start date of the period for which the consumed resources are to be returned. (required)
        :type start_date: datetime
        :param end_date: The end date of the period for which the consumed resources are to be returned. (required)
        :type end_date: datetime
        :param space: Specifies the ID of the space the operation should be executed in. (required)
        :type space: int
        :param expand:
        :type expand: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_spaces_consumed_resources_serialize(
            start_date=start_date,
            end_date=end_date,
            space=space,
            expand=expand,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "MetricUsageListResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_spaces_consumed_resources_without_preload_content(
        self,
        start_date: Annotated[datetime, Field(description="The start date of the period for which the consumed resources are to be returned.")],
        end_date: Annotated[datetime, Field(description="The end date of the period for which the consumed resources are to be returned.")],
        space: Annotated[StrictInt, Field(description="Specifies the ID of the space the operation should be executed in.")],
        expand: Optional[List[StrictStr]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List consumed resources



        :param start_date: The start date of the period for which the consumed resources are to be returned. (required)
        :type start_date: datetime
        :param end_date: The end date of the period for which the consumed resources are to be returned. (required)
        :type end_date: datetime
        :param space: Specifies the ID of the space the operation should be executed in. (required)
        :type space: int
        :param expand:
        :type expand: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_spaces_consumed_resources_serialize(
            start_date=start_date,
            end_date=end_date,
            space=space,
            expand=expand,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "MetricUsageListResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _get_spaces_consumed_resources_serialize(
        self,
        start_date,
        end_date,
        space,
        expand,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'expand': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if start_date is not None:
            if isinstance(start_date, datetime):
                _query_params.append(
                    (
                        'startDate',
                        start_date.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('startDate', start_date))
            
        if end_date is not None:
            if isinstance(end_date, datetime):
                _query_params.append(
                    (
                        'endDate',
                        end_date.strftime(
                            self.api_client.configuration.datetime_format
                        )
                    )
                )
            else:
                _query_params.append(('endDate', end_date))
            
        if expand is not None:
            
            _query_params.append(('expand', expand))
            
        # process the header parameters
        if space is not None:
            _header_params['Space'] = space
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='GET',
            resource_path='/spaces/consumed-resources',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )






# coding: utf-8

"""
Wallee AG Python SDK

This library allows to interact with the Wallee AG payment service.

Copyright owner: Wallee AG
Website: https://en.wallee.com
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from typing_extensions import Annotated
from wallee.models.charge_attempt_environment import ChargeAttemptEnvironment
from wallee.models.payment_app_processor_state import PaymentAppProcessorState
from wallee.models.payment_processor_configuration import PaymentProcessorConfiguration
from typing import Optional, Set
from typing_extensions import Self

class PaymentAppProcessor(BaseModel):
    """
    PaymentAppProcessor
    """ # noqa: E501
    documentation_url: Optional[StrictStr] = Field(default=None, description="A URL pointing to the documentation that explains how to configure and use the processor.", alias="documentationUrl")
    configured_environment: Optional[ChargeAttemptEnvironment] = Field(default=None, alias="configuredEnvironment")
    external_id: Optional[Annotated[str, Field(strict=True, max_length=40)]] = Field(default=None, description="A client-generated nonce which uniquely identifies some action to be executed. Subsequent requests with the same external ID do not execute the action again, but return the original result.", alias="externalId")
    svg_icon: Optional[Annotated[str, Field(strict=True, max_length=10000)]] = Field(default=None, description="An SVG icon representing the processor, displayed to the user in the interface.", alias="svgIcon")
    updated_on: Optional[datetime] = Field(default=None, description="The date and time when the processor was last updated.", alias="updatedOn")
    usable_in_production: Optional[StrictBool] = Field(default=None, description="Whether the processor is fully prepared and available for handling transactions in a production environment.", alias="usableInProduction")
    created_on: Optional[datetime] = Field(default=None, description="The date and time when the processor was created.", alias="createdOn")
    version: Optional[StrictInt] = Field(default=None, description="The version is used for optimistic locking and incremented whenever the object is updated.")
    processor_configuration: Optional[PaymentProcessorConfiguration] = Field(default=None, alias="processorConfiguration")
    linked_space_id: Optional[StrictInt] = Field(default=None, description="The ID of the space this object belongs to.", alias="linkedSpaceId")
    usable_in_production_since: Optional[datetime] = Field(default=None, description="the date and time when the processor became fully usable and available for handling transactions in a production environment.", alias="usableInProductionSince")
    name: Optional[Annotated[str, Field(strict=True, max_length=100)]] = Field(default=None, description="The name used to identify the processor.")
    id: Optional[StrictInt] = Field(default=None, description="A unique identifier for the object.")
    installation_id: Optional[StrictInt] = Field(default=None, description="The installation ID identifies the Web App installation.", alias="installationId")
    production_mode_url: Optional[StrictStr] = Field(default=None, description="A URL pointing to the site where merchants can set up production mode for the processor.", alias="productionModeUrl")
    state: Optional[PaymentAppProcessorState] = None
    __properties: ClassVar[List[str]] = ["documentationUrl", "configuredEnvironment", "externalId", "svgIcon", "updatedOn", "usableInProduction", "createdOn", "version", "processorConfiguration", "linkedSpaceId", "usableInProductionSince", "name", "id", "installationId", "productionModeUrl", "state"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PaymentAppProcessor from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "documentation_url",
            "external_id",
            "svg_icon",
            "updated_on",
            "usable_in_production",
            "created_on",
            "version",
            "linked_space_id",
            "usable_in_production_since",
            "name",
            "id",
            "installation_id",
            "production_mode_url",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of processor_configuration
        if self.processor_configuration:
            _dict['processorConfiguration'] = self.processor_configuration.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PaymentAppProcessor from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "documentationUrl": obj.get("documentationUrl"),
            "configuredEnvironment": obj.get("configuredEnvironment"),
            "externalId": obj.get("externalId"),
            "svgIcon": obj.get("svgIcon"),
            "updatedOn": obj.get("updatedOn"),
            "usableInProduction": obj.get("usableInProduction"),
            "createdOn": obj.get("createdOn"),
            "version": obj.get("version"),
            "processorConfiguration": PaymentProcessorConfiguration.from_dict(obj["processorConfiguration"]) if obj.get("processorConfiguration") is not None else None,
            "linkedSpaceId": obj.get("linkedSpaceId"),
            "usableInProductionSince": obj.get("usableInProductionSince"),
            "name": obj.get("name"),
            "id": obj.get("id"),
            "installationId": obj.get("installationId"),
            "productionModeUrl": obj.get("productionModeUrl"),
            "state": obj.get("state")
        })
        return _obj





# flake8: noqa

# import apis into api package
from wallee.service.accounts_service import AccountsService
from wallee.service.analytics_queries_service import AnalyticsQueriesService
from wallee.service.application_users_service import ApplicationUsersService
from wallee.service.application_users_roles_service import ApplicationUsersRolesService
from wallee.service.bank_accounts_service import BankAccountsService
from wallee.service.bank_transactions_service import BankTransactionsService
from wallee.service.bogus_express_checkout_service import BogusExpressCheckoutService
from wallee.service.charge_attempts_service import ChargeAttemptsService
from wallee.service.charge_bank_transactions_service import ChargeBankTransactionsService
from wallee.service.charge_flow_levels_service import ChargeFlowLevelsService
from wallee.service.charge_flows_service import ChargeFlowsService
from wallee.service.charge_flows_level_payment_links_service import ChargeFlowsLevelPaymentLinksService
from wallee.service.condition_types_service import ConditionTypesService
from wallee.service.consumed_resources_service import ConsumedResourcesService
from wallee.service.countries_service import CountriesService
from wallee.service.currencies_service import CurrenciesService
from wallee.service.currency_bank_accounts_service import CurrencyBankAccountsService
from wallee.service.customer_addresses_service import CustomerAddressesService
from wallee.service.customer_comments_service import CustomerCommentsService
from wallee.service.customers_service import CustomersService
from wallee.service.debt_collection_cases_service import DebtCollectionCasesService
from wallee.service.debt_collector_configurations_service import DebtCollectorConfigurationsService
from wallee.service.debt_collectors_service import DebtCollectorsService
from wallee.service.delivery_indications_service import DeliveryIndicationsService
from wallee.service.document_template_types_service import DocumentTemplateTypesService
from wallee.service.document_templates_service import DocumentTemplatesService
from wallee.service.dunning_cases_service import DunningCasesService
from wallee.service.dunning_flow_levels_service import DunningFlowLevelsService
from wallee.service.dunning_flows_service import DunningFlowsService
from wallee.service.express_checkout_service import ExpressCheckoutService
from wallee.service.external_transfer_bank_transactions_service import ExternalTransferBankTransactionsService
from wallee.service.human_users_service import HumanUsersService
from wallee.service.human_users_roles_service import HumanUsersRolesService
from wallee.service.internal_transfer_bank_transactions_service import InternalTransferBankTransactionsService
from wallee.service.label_descriptors_service import LabelDescriptorsService
from wallee.service.languages_service import LanguagesService
from wallee.service.legal_organization_forms_service import LegalOrganizationFormsService
from wallee.service.manual_tasks_service import ManualTasksService
from wallee.service.payment_connector_configurations_service import PaymentConnectorConfigurationsService
from wallee.service.payment_connectors_service import PaymentConnectorsService
from wallee.service.payment_links_service import PaymentLinksService
from wallee.service.payment_method_brands_service import PaymentMethodBrandsService
from wallee.service.payment_method_configurations_service import PaymentMethodConfigurationsService
from wallee.service.payment_methods_service import PaymentMethodsService
from wallee.service.payment_processor_configurations_service import PaymentProcessorConfigurationsService
from wallee.service.payment_processors_service import PaymentProcessorsService
from wallee.service.payment_sales_channels_service import PaymentSalesChannelsService
from wallee.service.payment_terminal_transaction_summaries_service import PaymentTerminalTransactionSummariesService
from wallee.service.payment_terminals_service import PaymentTerminalsService
from wallee.service.payment_web_apps_service import PaymentWebAppsService
from wallee.service.permissions_service import PermissionsService
from wallee.service.refund_bank_transactions_service import RefundBankTransactionsService
from wallee.service.refund_comments_service import RefundCommentsService
from wallee.service.refund_recovery_bank_transactions_service import RefundRecoveryBankTransactionsService
from wallee.service.refunds_service import RefundsService
from wallee.service.roles_service import RolesService
from wallee.service.single_sign_on_users_service import SingleSignOnUsersService
from wallee.service.single_sign_on_users_roles_service import SingleSignOnUsersRolesService
from wallee.service.spaces_service import SpacesService
from wallee.service.static_values_service import StaticValuesService
from wallee.service.subscribers_service import SubscribersService
from wallee.service.subscription_affiliates_service import SubscriptionAffiliatesService
from wallee.service.subscription_charges_service import SubscriptionChargesService
from wallee.service.subscription_ledger_entries_service import SubscriptionLedgerEntriesService
from wallee.service.subscription_metric_usage_reports_service import SubscriptionMetricUsageReportsService
from wallee.service.subscription_metrics_service import SubscriptionMetricsService
from wallee.service.subscription_period_bills_service import SubscriptionPeriodBillsService
from wallee.service.subscription_product_component_groups_service import SubscriptionProductComponentGroupsService
from wallee.service.subscription_product_components_service import SubscriptionProductComponentsService
from wallee.service.subscription_product_metered_fee_tiers_service import SubscriptionProductMeteredFeeTiersService
from wallee.service.subscription_product_metered_fees_service import SubscriptionProductMeteredFeesService
from wallee.service.subscription_product_period_fees_service import SubscriptionProductPeriodFeesService
from wallee.service.subscription_product_retirements_service import SubscriptionProductRetirementsService
from wallee.service.subscription_product_setup_fees_service import SubscriptionProductSetupFeesService
from wallee.service.subscription_product_version_retirements_service import SubscriptionProductVersionRetirementsService
from wallee.service.subscription_product_versions_service import SubscriptionProductVersionsService
from wallee.service.subscription_products_service import SubscriptionProductsService
from wallee.service.subscription_suspensions_service import SubscriptionSuspensionsService
from wallee.service.subscription_versions_service import SubscriptionVersionsService
from wallee.service.subscriptions_service import SubscriptionsService
from wallee.service.token_versions_service import TokenVersionsService
from wallee.service.tokens_service import TokensService
from wallee.service.transaction_client_platforms_service import TransactionClientPlatformsService
from wallee.service.transaction_comments_service import TransactionCommentsService
from wallee.service.transaction_completions_service import TransactionCompletionsService
from wallee.service.transaction_invoice_comments_service import TransactionInvoiceCommentsService
from wallee.service.transaction_invoices_service import TransactionInvoicesService
from wallee.service.transaction_line_item_versions_service import TransactionLineItemVersionsService
from wallee.service.transaction_voids_service import TransactionVoidsService
from wallee.service.transactions_service import TransactionsService
from wallee.service.web_apps_service import WebAppsService
from wallee.service.webhook_encryption_keys_service import WebhookEncryptionKeysService
from wallee.service.webhook_listeners_service import WebhookListenersService
from wallee.service.webhook_urls_service import WebhookURLsService




# coding: utf-8

# flake8: noqa
"""
Wallee AG Python SDK

This library allows to interact with the Wallee AG payment service.

Copyright owner: Wallee AG
Website: https://en.wallee.com
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


# import models into model package
from wallee.models.abstract_account_update import AbstractAccountUpdate
from wallee.models.abstract_application_user_update import AbstractApplicationUserUpdate
from wallee.models.abstract_customer_active import AbstractCustomerActive
from wallee.models.abstract_customer_address_active import AbstractCustomerAddressActive
from wallee.models.abstract_customer_comment_active import AbstractCustomerCommentActive
from wallee.models.abstract_debt_collection_case_update import AbstractDebtCollectionCaseUpdate
from wallee.models.abstract_debt_collector_configuration_update import AbstractDebtCollectorConfigurationUpdate
from wallee.models.abstract_human_user_update import AbstractHumanUserUpdate
from wallee.models.abstract_payment_connector_configuration_update import AbstractPaymentConnectorConfigurationUpdate
from wallee.models.abstract_payment_link_update import AbstractPaymentLinkUpdate
from wallee.models.abstract_payment_method_configuration_update import AbstractPaymentMethodConfigurationUpdate
from wallee.models.abstract_payment_processor_configuration_active import AbstractPaymentProcessorConfigurationActive
from wallee.models.abstract_payment_terminal_update import AbstractPaymentTerminalUpdate
from wallee.models.abstract_refund_comment_active import AbstractRefundCommentActive
from wallee.models.abstract_role_update import AbstractRoleUpdate
from wallee.models.abstract_single_sign_on_user_update import AbstractSingleSignOnUserUpdate
from wallee.models.abstract_space_update import AbstractSpaceUpdate
from wallee.models.abstract_subscriber_update import AbstractSubscriberUpdate
from wallee.models.abstract_subscription_affiliate_update import AbstractSubscriptionAffiliateUpdate
from wallee.models.abstract_subscription_metric_update import AbstractSubscriptionMetricUpdate
from wallee.models.abstract_subscription_product_active import AbstractSubscriptionProductActive
from wallee.models.abstract_token_update import AbstractTokenUpdate
from wallee.models.abstract_transaction_comment_active import AbstractTransactionCommentActive
from wallee.models.abstract_transaction_invoice_comment_active import AbstractTransactionInvoiceCommentActive
from wallee.models.abstract_transaction_pending import AbstractTransactionPending
from wallee.models.abstract_webhook_listener_update import AbstractWebhookListenerUpdate
from wallee.models.abstract_webhook_url_update import AbstractWebhookUrlUpdate
from wallee.models.account import Account
from wallee.models.account_create import AccountCreate
from wallee.models.account_list_response import AccountListResponse
from wallee.models.account_search_response import AccountSearchResponse
from wallee.models.account_state import AccountState
from wallee.models.account_type import AccountType
from wallee.models.account_update import AccountUpdate
from wallee.models.address import Address
from wallee.models.address_create import AddressCreate
from wallee.models.analytics_query_execution_request import AnalyticsQueryExecutionRequest
from wallee.models.analytics_query_execution_response import AnalyticsQueryExecutionResponse
from wallee.models.application_key_state import ApplicationKeyState
from wallee.models.application_user import ApplicationUser
from wallee.models.application_user_create import ApplicationUserCreate
from wallee.models.application_user_create_with_mac_key import ApplicationUserCreateWithMacKey
from wallee.models.application_user_list_response import ApplicationUserListResponse
from wallee.models.application_user_search_response import ApplicationUserSearchResponse
from wallee.models.application_user_update import ApplicationUserUpdate
from wallee.models.authenticated_card_data import AuthenticatedCardData
from wallee.models.authenticated_card_data_create import AuthenticatedCardDataCreate
from wallee.models.authenticated_card_request import AuthenticatedCardRequest
from wallee.models.bank_account import BankAccount
from wallee.models.bank_account_environment import BankAccountEnvironment
from wallee.models.bank_account_list_response import BankAccountListResponse
from wallee.models.bank_account_search_response import BankAccountSearchResponse
from wallee.models.bank_account_state import BankAccountState
from wallee.models.bank_account_type import BankAccountType
from wallee.models.bank_transaction import BankTransaction
from wallee.models.bank_transaction_flow_direction import BankTransactionFlowDirection
from wallee.models.bank_transaction_list_response import BankTransactionListResponse
from wallee.models.bank_transaction_search_response import BankTransactionSearchResponse
from wallee.models.bank_transaction_source import BankTransactionSource
from wallee.models.bank_transaction_state import BankTransactionState
from wallee.models.bank_transaction_type import BankTransactionType
from wallee.models.billing_cycle_model import BillingCycleModel
from wallee.models.billing_cycle_type import BillingCycleType
from wallee.models.billing_day_customization import BillingDayCustomization
from wallee.models.bogus_express_checkout_approval_request import BogusExpressCheckoutApprovalRequest
from wallee.models.bogus_express_checkout_payment_data import BogusExpressCheckoutPaymentData
from wallee.models.card_authentication_response import CardAuthenticationResponse
from wallee.models.card_authentication_version import CardAuthenticationVersion
from wallee.models.card_cryptogram import CardCryptogram
from wallee.models.card_cryptogram_create import CardCryptogramCreate
from wallee.models.cardholder_authentication import CardholderAuthentication
from wallee.models.cardholder_authentication_create import CardholderAuthenticationCreate
from wallee.models.charge import Charge
from wallee.models.charge_attempt import ChargeAttempt
from wallee.models.charge_attempt_environment import ChargeAttemptEnvironment
from wallee.models.charge_attempt_list_response import ChargeAttemptListResponse
from wallee.models.charge_attempt_search_response import ChargeAttemptSearchResponse
from wallee.models.charge_attempt_state import ChargeAttemptState
from wallee.models.charge_bank_transaction import ChargeBankTransaction
from wallee.models.charge_bank_transaction_list_response import ChargeBankTransactionListResponse
from wallee.models.charge_bank_transaction_search_response import ChargeBankTransactionSearchResponse
from wallee.models.charge_flow import ChargeFlow
from wallee.models.charge_flow_level import ChargeFlowLevel
from wallee.models.charge_flow_level_configuration import ChargeFlowLevelConfiguration
from wallee.models.charge_flow_level_configuration_type import ChargeFlowLevelConfigurationType
from wallee.models.charge_flow_level_list_response import ChargeFlowLevelListResponse
from wallee.models.charge_flow_level_payment_link import ChargeFlowLevelPaymentLink
from wallee.models.charge_flow_level_payment_link_list_response import ChargeFlowLevelPaymentLinkListResponse
from wallee.models.charge_flow_level_payment_link_search_response import ChargeFlowLevelPaymentLinkSearchResponse
from wallee.models.charge_flow_level_search_response import ChargeFlowLevelSearchResponse
from wallee.models.charge_flow_level_state import ChargeFlowLevelState
from wallee.models.charge_flow_list_response import ChargeFlowListResponse
from wallee.models.charge_flow_search_response import ChargeFlowSearchResponse
from wallee.models.charge_state import ChargeState
from wallee.models.charge_type import ChargeType
from wallee.models.client_platform_information_list_response import ClientPlatformInformationListResponse
from wallee.models.client_platform_information_search_response import ClientPlatformInformationSearchResponse
from wallee.models.completion_line_item import CompletionLineItem
from wallee.models.completion_line_item_create import CompletionLineItemCreate
from wallee.models.completion_list_response import CompletionListResponse
from wallee.models.completion_search_response import CompletionSearchResponse
from wallee.models.condition import Condition
from wallee.models.condition_type import ConditionType
from wallee.models.condition_type_list_response import ConditionTypeListResponse
from wallee.models.condition_type_search_response import ConditionTypeSearchResponse
from wallee.models.connector_invocation import ConnectorInvocation
from wallee.models.connector_invocation_stage import ConnectorInvocationStage
from wallee.models.country_list_response import CountryListResponse
from wallee.models.country_search_response import CountrySearchResponse
from wallee.models.creation_entity_state import CreationEntityState
from wallee.models.currency_bank_account import CurrencyBankAccount
from wallee.models.currency_bank_account_list_response import CurrencyBankAccountListResponse
from wallee.models.currency_bank_account_search_response import CurrencyBankAccountSearchResponse
from wallee.models.currency_list_response import CurrencyListResponse
from wallee.models.currency_search_response import CurrencySearchResponse
from wallee.models.customer import Customer
from wallee.models.customer_active import CustomerActive
from wallee.models.customer_address import CustomerAddress
from wallee.models.customer_address_active import CustomerAddressActive
from wallee.models.customer_address_create import CustomerAddressCreate
from wallee.models.customer_address_list_response import CustomerAddressListResponse
from wallee.models.customer_address_search_response import CustomerAddressSearchResponse
from wallee.models.customer_address_type import CustomerAddressType
from wallee.models.customer_comment import CustomerComment
from wallee.models.customer_comment_active import CustomerCommentActive
from wallee.models.customer_comment_create import CustomerCommentCreate
from wallee.models.customer_comment_list_response import CustomerCommentListResponse
from wallee.models.customer_comment_search_response import CustomerCommentSearchResponse
from wallee.models.customer_create import CustomerCreate
from wallee.models.customer_email_address_list_response import CustomerEmailAddressListResponse
from wallee.models.customer_list_response import CustomerListResponse
from wallee.models.customer_postal_address import CustomerPostalAddress
from wallee.models.customer_postal_address_create import CustomerPostalAddressCreate
from wallee.models.customer_search_response import CustomerSearchResponse
from wallee.models.customers_presence import CustomersPresence
from wallee.models.data_collection_type import DataCollectionType
from wallee.models.debt_collection_case import DebtCollectionCase
from wallee.models.debt_collection_case_create import DebtCollectionCaseCreate
from wallee.models.debt_collection_case_document import DebtCollectionCaseDocument
from wallee.models.debt_collection_case_document_list_response import DebtCollectionCaseDocumentListResponse
from wallee.models.debt_collection_case_list_response import DebtCollectionCaseListResponse
from wallee.models.debt_collection_case_search_response import DebtCollectionCaseSearchResponse
from wallee.models.debt_collection_case_source import DebtCollectionCaseSource
from wallee.models.debt_collection_case_state import DebtCollectionCaseState
from wallee.models.debt_collection_case_update import DebtCollectionCaseUpdate
from wallee.models.debt_collection_environment import DebtCollectionEnvironment
from wallee.models.debt_collection_receipt import DebtCollectionReceipt
from wallee.models.debt_collection_receipt_source import DebtCollectionReceiptSource
from wallee.models.debt_collector import DebtCollector
from wallee.models.debt_collector_condition import DebtCollectorCondition
from wallee.models.debt_collector_condition_type import DebtCollectorConditionType
from wallee.models.debt_collector_configuration import DebtCollectorConfiguration
from wallee.models.debt_collector_configuration_create import DebtCollectorConfigurationCreate
from wallee.models.debt_collector_configuration_list_response import DebtCollectorConfigurationListResponse
from wallee.models.debt_collector_configuration_search_response import DebtCollectorConfigurationSearchResponse
from wallee.models.debt_collector_configuration_update import DebtCollectorConfigurationUpdate
from wallee.models.debt_collector_list_response import DebtCollectorListResponse
from wallee.models.debt_collector_search_response import DebtCollectorSearchResponse
from wallee.models.delivery_indication import DeliveryIndication
from wallee.models.delivery_indication_decision_reason import DeliveryIndicationDecisionReason
from wallee.models.delivery_indication_list_response import DeliveryIndicationListResponse
from wallee.models.delivery_indication_search_response import DeliveryIndicationSearchResponse
from wallee.models.delivery_indication_state import DeliveryIndicationState
from wallee.models.displayable_day_of_week import DisplayableDayOfWeek
from wallee.models.displayable_month import DisplayableMonth
from wallee.models.document_template import DocumentTemplate
from wallee.models.document_template_list_response import DocumentTemplateListResponse
from wallee.models.document_template_search_response import DocumentTemplateSearchResponse
from wallee.models.document_template_type import DocumentTemplateType
from wallee.models.document_template_type_group import DocumentTemplateTypeGroup
from wallee.models.document_template_type_list_response import DocumentTemplateTypeListResponse
from wallee.models.document_template_type_search_response import DocumentTemplateTypeSearchResponse
from wallee.models.dunning_case import DunningCase
from wallee.models.dunning_case_list_response import DunningCaseListResponse
from wallee.models.dunning_case_search_response import DunningCaseSearchResponse
from wallee.models.dunning_case_state import DunningCaseState
from wallee.models.dunning_condition import DunningCondition
from wallee.models.dunning_condition_type import DunningConditionType
from wallee.models.dunning_flow import DunningFlow
from wallee.models.dunning_flow_level import DunningFlowLevel
from wallee.models.dunning_flow_level_list_response import DunningFlowLevelListResponse
from wallee.models.dunning_flow_level_processor import DunningFlowLevelProcessor
from wallee.models.dunning_flow_level_search_response import DunningFlowLevelSearchResponse
from wallee.models.dunning_flow_list_response import DunningFlowListResponse
from wallee.models.dunning_flow_search_response import DunningFlowSearchResponse
from wallee.models.dunning_flow_type import DunningFlowType
from wallee.models.environment import Environment
from wallee.models.express_checkout_approval_response import ExpressCheckoutApprovalResponse
from wallee.models.express_checkout_create_response import ExpressCheckoutCreateResponse
from wallee.models.express_checkout_session import ExpressCheckoutSession
from wallee.models.express_checkout_session_create import ExpressCheckoutSessionCreate
from wallee.models.express_checkout_session_state import ExpressCheckoutSessionState
from wallee.models.express_checkout_shipping_address_change_request import ExpressCheckoutShippingAddressChangeRequest
from wallee.models.express_checkout_shipping_address_change_response import ExpressCheckoutShippingAddressChangeResponse
from wallee.models.express_checkout_shipping_method_change_request import ExpressCheckoutShippingMethodChangeRequest
from wallee.models.express_checkout_shipping_method_change_response import ExpressCheckoutShippingMethodChangeResponse
from wallee.models.express_checkout_shipping_option import ExpressCheckoutShippingOption
from wallee.models.express_checkout_wallet_type import ExpressCheckoutWalletType
from wallee.models.external_transfer_bank_transaction import ExternalTransferBankTransaction
from wallee.models.external_transfer_bank_transaction_list_response import ExternalTransferBankTransactionListResponse
from wallee.models.external_transfer_bank_transaction_search_response import ExternalTransferBankTransactionSearchResponse
from wallee.models.facade_user_friendly_query_status_model import FacadeUserFriendlyQueryStatusModel
from wallee.models.failure_category import FailureCategory
from wallee.models.failure_reason import FailureReason
from wallee.models.feature import Feature
from wallee.models.feature_category import FeatureCategory
from wallee.models.gender import Gender
from wallee.models.human_user import HumanUser
from wallee.models.human_user_create import HumanUserCreate
from wallee.models.human_user_list_response import HumanUserListResponse
from wallee.models.human_user_search_response import HumanUserSearchResponse
from wallee.models.human_user_update import HumanUserUpdate
from wallee.models.internal_transfer_bank_transaction import InternalTransferBankTransaction
from wallee.models.internal_transfer_bank_transaction_list_response import InternalTransferBankTransactionListResponse
from wallee.models.internal_transfer_bank_transaction_search_response import InternalTransferBankTransactionSearchResponse
from wallee.models.invoice_comment_list_response import InvoiceCommentListResponse
from wallee.models.invoice_comment_search_response import InvoiceCommentSearchResponse
from wallee.models.invoice_list_response import InvoiceListResponse
from wallee.models.invoice_search_response import InvoiceSearchResponse
from wallee.models.label import Label
from wallee.models.label_descriptor import LabelDescriptor
from wallee.models.label_descriptor_category import LabelDescriptorCategory
from wallee.models.label_descriptor_group import LabelDescriptorGroup
from wallee.models.label_descriptor_group_list_response import LabelDescriptorGroupListResponse
from wallee.models.label_descriptor_group_search_response import LabelDescriptorGroupSearchResponse
from wallee.models.label_descriptor_list_response import LabelDescriptorListResponse
from wallee.models.label_descriptor_search_response import LabelDescriptorSearchResponse
from wallee.models.label_descriptor_type import LabelDescriptorType
from wallee.models.language_list_response import LanguageListResponse
from wallee.models.language_search_response import LanguageSearchResponse
from wallee.models.legal_organization_form import LegalOrganizationForm
from wallee.models.legal_organization_form_list_response import LegalOrganizationFormListResponse
from wallee.models.legal_organization_form_search_response import LegalOrganizationFormSearchResponse
from wallee.models.line_item import LineItem
from wallee.models.line_item_attribute import LineItemAttribute
from wallee.models.line_item_attribute_create import LineItemAttributeCreate
from wallee.models.line_item_create import LineItemCreate
from wallee.models.line_item_reduction import LineItemReduction
from wallee.models.line_item_reduction_create import LineItemReductionCreate
from wallee.models.line_item_type import LineItemType
from wallee.models.line_item_version_list_response import LineItemVersionListResponse
from wallee.models.line_item_version_search_response import LineItemVersionSearchResponse
from wallee.models.localized_string import LocalizedString
from wallee.models.manual_task import ManualTask
from wallee.models.manual_task_action import ManualTaskAction
from wallee.models.manual_task_action_style import ManualTaskActionStyle
from wallee.models.manual_task_list_response import ManualTaskListResponse
from wallee.models.manual_task_search_response import ManualTaskSearchResponse
from wallee.models.manual_task_state import ManualTaskState
from wallee.models.manual_task_type import ManualTaskType
from wallee.models.metric_list_response import MetricListResponse
from wallee.models.metric_search_response import MetricSearchResponse
from wallee.models.metric_usage import MetricUsage
from wallee.models.metric_usage_list_response import MetricUsageListResponse
from wallee.models.metric_usage_report_list_response import MetricUsageReportListResponse
from wallee.models.metric_usage_report_search_response import MetricUsageReportSearchResponse
from wallee.models.one_click_payment_mode import OneClickPaymentMode
from wallee.models.pan_type import PanType
from wallee.models.payment_adjustment import PaymentAdjustment
from wallee.models.payment_adjustment_type import PaymentAdjustmentType
from wallee.models.payment_app_charge_attempt_target_state import PaymentAppChargeAttemptTargetState
from wallee.models.payment_app_charge_attempt_update import PaymentAppChargeAttemptUpdate
from wallee.models.payment_app_completion_configuration import PaymentAppCompletionConfiguration
from wallee.models.payment_app_completion_configuration_create import PaymentAppCompletionConfigurationCreate
from wallee.models.payment_app_completion_target_state import PaymentAppCompletionTargetState
from wallee.models.payment_app_completion_update import PaymentAppCompletionUpdate
from wallee.models.payment_app_connector import PaymentAppConnector
from wallee.models.payment_app_connector_details import PaymentAppConnectorDetails
from wallee.models.payment_app_connector_details_create import PaymentAppConnectorDetailsCreate
from wallee.models.payment_app_connector_state import PaymentAppConnectorState
from wallee.models.payment_app_processor import PaymentAppProcessor
from wallee.models.payment_app_processor_details import PaymentAppProcessorDetails
from wallee.models.payment_app_processor_details_create import PaymentAppProcessorDetailsCreate
from wallee.models.payment_app_processor_state import PaymentAppProcessorState
from wallee.models.payment_app_refund_configuration import PaymentAppRefundConfiguration
from wallee.models.payment_app_refund_configuration_create import PaymentAppRefundConfigurationCreate
from wallee.models.payment_app_refund_target_state import PaymentAppRefundTargetState
from wallee.models.payment_app_refund_update import PaymentAppRefundUpdate
from wallee.models.payment_app_void_target_state import PaymentAppVoidTargetState
from wallee.models.payment_app_void_update import PaymentAppVoidUpdate
from wallee.models.payment_connector import PaymentConnector
from wallee.models.payment_connector_configuration import PaymentConnectorConfiguration
from wallee.models.payment_connector_configuration_create import PaymentConnectorConfigurationCreate
from wallee.models.payment_connector_configuration_list_response import PaymentConnectorConfigurationListResponse
from wallee.models.payment_connector_configuration_search_response import PaymentConnectorConfigurationSearchResponse
from wallee.models.payment_connector_configuration_update import PaymentConnectorConfigurationUpdate
from wallee.models.payment_connector_feature import PaymentConnectorFeature
from wallee.models.payment_connector_list_response import PaymentConnectorListResponse
from wallee.models.payment_connector_search_response import PaymentConnectorSearchResponse
from wallee.models.payment_contract import PaymentContract
from wallee.models.payment_contract_state import PaymentContractState
from wallee.models.payment_contract_type import PaymentContractType
from wallee.models.payment_information_hash import PaymentInformationHash
from wallee.models.payment_information_hash_type import PaymentInformationHashType
from wallee.models.payment_link import PaymentLink
from wallee.models.payment_link_active import PaymentLinkActive
from wallee.models.payment_link_address_handling_mode import PaymentLinkAddressHandlingMode
from wallee.models.payment_link_create import PaymentLinkCreate
from wallee.models.payment_link_list_response import PaymentLinkListResponse
from wallee.models.payment_link_protection_mode import PaymentLinkProtectionMode
from wallee.models.payment_link_search_response import PaymentLinkSearchResponse
from wallee.models.payment_link_update import PaymentLinkUpdate
from wallee.models.payment_method import PaymentMethod
from wallee.models.payment_method_brand import PaymentMethodBrand
from wallee.models.payment_method_brand_list_response import PaymentMethodBrandListResponse
from wallee.models.payment_method_brand_search_response import PaymentMethodBrandSearchResponse
from wallee.models.payment_method_configuration import PaymentMethodConfiguration
from wallee.models.payment_method_configuration_active import PaymentMethodConfigurationActive
from wallee.models.payment_method_configuration_create import PaymentMethodConfigurationCreate
from wallee.models.payment_method_configuration_list_response import PaymentMethodConfigurationListResponse
from wallee.models.payment_method_configuration_search_response import PaymentMethodConfigurationSearchResponse
from wallee.models.payment_method_configuration_update import PaymentMethodConfigurationUpdate
from wallee.models.payment_method_list_response import PaymentMethodListResponse
from wallee.models.payment_method_search_response import PaymentMethodSearchResponse
from wallee.models.payment_primary_risk_taker import PaymentPrimaryRiskTaker
from wallee.models.payment_processor import PaymentProcessor
from wallee.models.payment_processor_configuration import PaymentProcessorConfiguration
from wallee.models.payment_processor_configuration_active import PaymentProcessorConfigurationActive
from wallee.models.payment_processor_configuration_create import PaymentProcessorConfigurationCreate
from wallee.models.payment_processor_configuration_list_response import PaymentProcessorConfigurationListResponse
from wallee.models.payment_processor_configuration_search_response import PaymentProcessorConfigurationSearchResponse
from wallee.models.payment_processor_list_response import PaymentProcessorListResponse
from wallee.models.payment_processor_search_response import PaymentProcessorSearchResponse
from wallee.models.payment_terminal import PaymentTerminal
from wallee.models.payment_terminal_address import PaymentTerminalAddress
from wallee.models.payment_terminal_configuration import PaymentTerminalConfiguration
from wallee.models.payment_terminal_configuration_state import PaymentTerminalConfigurationState
from wallee.models.payment_terminal_configuration_version import PaymentTerminalConfigurationVersion
from wallee.models.payment_terminal_configuration_version_state import PaymentTerminalConfigurationVersionState
from wallee.models.payment_terminal_create import PaymentTerminalCreate
from wallee.models.payment_terminal_dcc_transaction_sum import PaymentTerminalDccTransactionSum
from wallee.models.payment_terminal_location import PaymentTerminalLocation
from wallee.models.payment_terminal_location_state import PaymentTerminalLocationState
from wallee.models.payment_terminal_location_version import PaymentTerminalLocationVersion
from wallee.models.payment_terminal_location_version_state import PaymentTerminalLocationVersionState
from wallee.models.payment_terminal_preparing import PaymentTerminalPreparing
from wallee.models.payment_terminal_receipt_type import PaymentTerminalReceiptType
from wallee.models.payment_terminal_state import PaymentTerminalState
from wallee.models.payment_terminal_transaction_sum import PaymentTerminalTransactionSum
from wallee.models.payment_terminal_transaction_summary import PaymentTerminalTransactionSummary
from wallee.models.payment_terminal_transaction_summary_reference import PaymentTerminalTransactionSummaryReference
from wallee.models.payment_terminal_type import PaymentTerminalType
from wallee.models.payment_terminal_update import PaymentTerminalUpdate
from wallee.models.permission import Permission
from wallee.models.permission_list_response import PermissionListResponse
from wallee.models.permission_search_response import PermissionSearchResponse
from wallee.models.persistable_currency_amount import PersistableCurrencyAmount
from wallee.models.persistable_currency_amount_update import PersistableCurrencyAmountUpdate
from wallee.models.product_component_group_list_response import ProductComponentGroupListResponse
from wallee.models.product_component_group_search_response import ProductComponentGroupSearchResponse
from wallee.models.product_component_list_response import ProductComponentListResponse
from wallee.models.product_component_search_response import ProductComponentSearchResponse
from wallee.models.product_fee_type import ProductFeeType
from wallee.models.product_list_response import ProductListResponse
from wallee.models.product_metered_fee import ProductMeteredFee
from wallee.models.product_metered_fee_list_response import ProductMeteredFeeListResponse
from wallee.models.product_metered_fee_search_response import ProductMeteredFeeSearchResponse
from wallee.models.product_metered_fee_tier_list_response import ProductMeteredFeeTierListResponse
from wallee.models.product_metered_fee_tier_search_response import ProductMeteredFeeTierSearchResponse
from wallee.models.product_metered_fee_update import ProductMeteredFeeUpdate
from wallee.models.product_metered_tier_fee import ProductMeteredTierFee
from wallee.models.product_metered_tier_fee_update import ProductMeteredTierFeeUpdate
from wallee.models.product_metered_tier_pricing import ProductMeteredTierPricing
from wallee.models.product_period_fee import ProductPeriodFee
from wallee.models.product_period_fee_list_response import ProductPeriodFeeListResponse
from wallee.models.product_period_fee_search_response import ProductPeriodFeeSearchResponse
from wallee.models.product_period_fee_update import ProductPeriodFeeUpdate
from wallee.models.product_retirement_list_response import ProductRetirementListResponse
from wallee.models.product_retirement_search_response import ProductRetirementSearchResponse
from wallee.models.product_search_response import ProductSearchResponse
from wallee.models.product_setup_fee import ProductSetupFee
from wallee.models.product_setup_fee_list_response import ProductSetupFeeListResponse
from wallee.models.product_setup_fee_search_response import ProductSetupFeeSearchResponse
from wallee.models.product_setup_fee_update import ProductSetupFeeUpdate
from wallee.models.product_version_list_response import ProductVersionListResponse
from wallee.models.product_version_retirement_list_response import ProductVersionRetirementListResponse
from wallee.models.product_version_retirement_search_response import ProductVersionRetirementSearchResponse
from wallee.models.product_version_search_response import ProductVersionSearchResponse
from wallee.models.recurring_indicator import RecurringIndicator
from wallee.models.refund import Refund
from wallee.models.refund_bank_transaction import RefundBankTransaction
from wallee.models.refund_bank_transaction_list_response import RefundBankTransactionListResponse
from wallee.models.refund_bank_transaction_search_response import RefundBankTransactionSearchResponse
from wallee.models.refund_comment import RefundComment
from wallee.models.refund_comment_active import RefundCommentActive
from wallee.models.refund_comment_create import RefundCommentCreate
from wallee.models.refund_comment_list_response import RefundCommentListResponse
from wallee.models.refund_comment_search_response import RefundCommentSearchResponse
from wallee.models.refund_create import RefundCreate
from wallee.models.refund_list_response import RefundListResponse
from wallee.models.refund_recovery_bank_transaction import RefundRecoveryBankTransaction
from wallee.models.refund_recovery_bank_transaction_list_response import RefundRecoveryBankTransactionListResponse
from wallee.models.refund_recovery_bank_transaction_search_response import RefundRecoveryBankTransactionSearchResponse
from wallee.models.refund_search_response import RefundSearchResponse
from wallee.models.refund_state import RefundState
from wallee.models.refund_type import RefundType
from wallee.models.rendered_document import RenderedDocument
from wallee.models.rendered_terminal_receipt import RenderedTerminalReceipt
from wallee.models.rendered_terminal_receipt_list_response import RenderedTerminalReceiptListResponse
from wallee.models.rendered_terminal_transaction_summary import RenderedTerminalTransactionSummary
from wallee.models.rest_address_format import RestAddressFormat
from wallee.models.rest_address_format_field import RestAddressFormatField
from wallee.models.rest_api_bulk_operation_result import RestApiBulkOperationResult
from wallee.models.rest_api_error_response import RestApiErrorResponse
from wallee.models.rest_application_user_mac_key import RestApplicationUserMacKey
from wallee.models.rest_application_user_mac_key_created import RestApplicationUserMacKeyCreated
from wallee.models.rest_country import RestCountry
from wallee.models.rest_country_state import RestCountryState
from wallee.models.rest_currency import RestCurrency
from wallee.models.rest_customer_email_address import RestCustomerEmailAddress
from wallee.models.rest_language import RestLanguage
from wallee.models.result_portion_model import ResultPortionModel
from wallee.models.role import Role
from wallee.models.role_create import RoleCreate
from wallee.models.role_list_response import RoleListResponse
from wallee.models.role_search_response import RoleSearchResponse
from wallee.models.role_state import RoleState
from wallee.models.role_update import RoleUpdate
from wallee.models.sales_channel import SalesChannel
from wallee.models.sales_channel_list_response import SalesChannelListResponse
from wallee.models.sales_channel_search_response import SalesChannelSearchResponse
from wallee.models.scope import Scope
from wallee.models.scope_single_sign_on_provider import ScopeSingleSignOnProvider
from wallee.models.single_sign_on_user import SingleSignOnUser
from wallee.models.single_sign_on_user_create import SingleSignOnUserCreate
from wallee.models.single_sign_on_user_list_response import SingleSignOnUserListResponse
from wallee.models.single_sign_on_user_search_response import SingleSignOnUserSearchResponse
from wallee.models.single_sign_on_user_update import SingleSignOnUserUpdate
from wallee.models.sorting_order import SortingOrder
from wallee.models.space import Space
from wallee.models.space_address import SpaceAddress
from wallee.models.space_address_create import SpaceAddressCreate
from wallee.models.space_create import SpaceCreate
from wallee.models.space_list_response import SpaceListResponse
from wallee.models.space_search_response import SpaceSearchResponse
from wallee.models.space_update import SpaceUpdate
from wallee.models.space_view import SpaceView
from wallee.models.static_value import StaticValue
from wallee.models.static_value_list_response import StaticValueListResponse
from wallee.models.static_value_search_response import StaticValueSearchResponse
from wallee.models.submitted_analytics_query_execution import SubmittedAnalyticsQueryExecution
from wallee.models.subscriber import Subscriber
from wallee.models.subscriber_active import SubscriberActive
from wallee.models.subscriber_create import SubscriberCreate
from wallee.models.subscriber_list_response import SubscriberListResponse
from wallee.models.subscriber_search_response import SubscriberSearchResponse
from wallee.models.subscriber_update import SubscriberUpdate
from wallee.models.subscription import Subscription
from wallee.models.subscription_affiliate import SubscriptionAffiliate
from wallee.models.subscription_affiliate_create import SubscriptionAffiliateCreate
from wallee.models.subscription_affiliate_deleted import SubscriptionAffiliateDeleted
from wallee.models.subscription_affiliate_deleting import SubscriptionAffiliateDeleting
from wallee.models.subscription_affiliate_inactive import SubscriptionAffiliateInactive
from wallee.models.subscription_affiliate_list_response import SubscriptionAffiliateListResponse
from wallee.models.subscription_affiliate_search_response import SubscriptionAffiliateSearchResponse
from wallee.models.subscription_affiliate_update import SubscriptionAffiliateUpdate
from wallee.models.subscription_charge import SubscriptionCharge
from wallee.models.subscription_charge_create import SubscriptionChargeCreate
from wallee.models.subscription_charge_list_response import SubscriptionChargeListResponse
from wallee.models.subscription_charge_processing_type import SubscriptionChargeProcessingType
from wallee.models.subscription_charge_search_response import SubscriptionChargeSearchResponse
from wallee.models.subscription_charge_state import SubscriptionChargeState
from wallee.models.subscription_charge_type import SubscriptionChargeType
from wallee.models.subscription_component_configuration import SubscriptionComponentConfiguration
from wallee.models.subscription_component_reference_configuration import SubscriptionComponentReferenceConfiguration
from wallee.models.subscription_create_request import SubscriptionCreateRequest
from wallee.models.subscription_initialize_subscriber_present_request import SubscriptionInitializeSubscriberPresentRequest
from wallee.models.subscription_ledger_entry import SubscriptionLedgerEntry
from wallee.models.subscription_ledger_entry_create import SubscriptionLedgerEntryCreate
from wallee.models.subscription_ledger_entry_list_response import SubscriptionLedgerEntryListResponse
from wallee.models.subscription_ledger_entry_search_response import SubscriptionLedgerEntrySearchResponse
from wallee.models.subscription_ledger_entry_state import SubscriptionLedgerEntryState
from wallee.models.subscription_list_response import SubscriptionListResponse
from wallee.models.subscription_metric import SubscriptionMetric
from wallee.models.subscription_metric_active import SubscriptionMetricActive
from wallee.models.subscription_metric_create import SubscriptionMetricCreate
from wallee.models.subscription_metric_type import SubscriptionMetricType
from wallee.models.subscription_metric_update import SubscriptionMetricUpdate
from wallee.models.subscription_metric_usage_report import SubscriptionMetricUsageReport
from wallee.models.subscription_metric_usage_report_create import SubscriptionMetricUsageReportCreate
from wallee.models.subscription_pending import SubscriptionPending
from wallee.models.subscription_period_bill import SubscriptionPeriodBill
from wallee.models.subscription_period_bill_list_response import SubscriptionPeriodBillListResponse
from wallee.models.subscription_period_bill_search_response import SubscriptionPeriodBillSearchResponse
from wallee.models.subscription_period_bill_state import SubscriptionPeriodBillState
from wallee.models.subscription_product import SubscriptionProduct
from wallee.models.subscription_product_active import SubscriptionProductActive
from wallee.models.subscription_product_component import SubscriptionProductComponent
from wallee.models.subscription_product_component_group import SubscriptionProductComponentGroup
from wallee.models.subscription_product_component_group_update import SubscriptionProductComponentGroupUpdate
from wallee.models.subscription_product_component_reference import SubscriptionProductComponentReference
from wallee.models.subscription_product_component_reference_state import SubscriptionProductComponentReferenceState
from wallee.models.subscription_product_component_update import SubscriptionProductComponentUpdate
from wallee.models.subscription_product_create import SubscriptionProductCreate
from wallee.models.subscription_product_retirement import SubscriptionProductRetirement
from wallee.models.subscription_product_retirement_request import SubscriptionProductRetirementRequest
from wallee.models.subscription_product_state import SubscriptionProductState
from wallee.models.subscription_product_version import SubscriptionProductVersion
from wallee.models.subscription_product_version_pending import SubscriptionProductVersionPending
from wallee.models.subscription_product_version_retirement import SubscriptionProductVersionRetirement
from wallee.models.subscription_product_version_retirement_request import SubscriptionProductVersionRetirementRequest
from wallee.models.subscription_product_version_state import SubscriptionProductVersionState
from wallee.models.subscription_search_response import SubscriptionSearchResponse
from wallee.models.subscription_state import SubscriptionState
from wallee.models.subscription_suspension import SubscriptionSuspension
from wallee.models.subscription_suspension_action import SubscriptionSuspensionAction
from wallee.models.subscription_suspension_reason import SubscriptionSuspensionReason
from wallee.models.subscription_suspension_state import SubscriptionSuspensionState
from wallee.models.subscription_update import SubscriptionUpdate
from wallee.models.subscription_update_request import SubscriptionUpdateRequest
from wallee.models.subscription_version import SubscriptionVersion
from wallee.models.subscription_version_list_response import SubscriptionVersionListResponse
from wallee.models.subscription_version_search_response import SubscriptionVersionSearchResponse
from wallee.models.subscription_version_state import SubscriptionVersionState
from wallee.models.suspension_creation_request import SuspensionCreationRequest
from wallee.models.suspension_list_response import SuspensionListResponse
from wallee.models.suspension_search_response import SuspensionSearchResponse
from wallee.models.tax import Tax
from wallee.models.tax_calculation import TaxCalculation
from wallee.models.tax_class import TaxClass
from wallee.models.tax_create import TaxCreate
from wallee.models.tenant_database import TenantDatabase
from wallee.models.terminal_list_response import TerminalListResponse
from wallee.models.terminal_receipt_format import TerminalReceiptFormat
from wallee.models.terminal_search_response import TerminalSearchResponse
from wallee.models.terminal_transaction_summary_list_response import TerminalTransactionSummaryListResponse
from wallee.models.terminal_transaction_summary_search_response import TerminalTransactionSummarySearchResponse
from wallee.models.token import Token
from wallee.models.token_create import TokenCreate
from wallee.models.token_list_response import TokenListResponse
from wallee.models.token_search_response import TokenSearchResponse
from wallee.models.token_update import TokenUpdate
from wallee.models.token_version import TokenVersion
from wallee.models.token_version_list_response import TokenVersionListResponse
from wallee.models.token_version_retry_strategy import TokenVersionRetryStrategy
from wallee.models.token_version_search_response import TokenVersionSearchResponse
from wallee.models.token_version_state import TokenVersionState
from wallee.models.token_version_type import TokenVersionType
from wallee.models.tokenization_mode import TokenizationMode
from wallee.models.tokenized_card_data import TokenizedCardData
from wallee.models.tokenized_card_data_create import TokenizedCardDataCreate
from wallee.models.tokenized_card_request import TokenizedCardRequest
from wallee.models.transaction import Transaction
from wallee.models.transaction_client_platform_information import TransactionClientPlatformInformation
from wallee.models.transaction_comment import TransactionComment
from wallee.models.transaction_comment_active import TransactionCommentActive
from wallee.models.transaction_comment_create import TransactionCommentCreate
from wallee.models.transaction_comment_list_response import TransactionCommentListResponse
from wallee.models.transaction_comment_search_response import TransactionCommentSearchResponse
from wallee.models.transaction_completion import TransactionCompletion
from wallee.models.transaction_completion_behavior import TransactionCompletionBehavior
from wallee.models.transaction_completion_details import TransactionCompletionDetails
from wallee.models.transaction_completion_mode import TransactionCompletionMode
from wallee.models.transaction_completion_state import TransactionCompletionState
from wallee.models.transaction_create import TransactionCreate
from wallee.models.transaction_environment_selection_strategy import TransactionEnvironmentSelectionStrategy
from wallee.models.transaction_group import TransactionGroup
from wallee.models.transaction_group_state import TransactionGroupState
from wallee.models.transaction_invoice import TransactionInvoice
from wallee.models.transaction_invoice_comment import TransactionInvoiceComment
from wallee.models.transaction_invoice_comment_active import TransactionInvoiceCommentActive
from wallee.models.transaction_invoice_comment_create import TransactionInvoiceCommentCreate
from wallee.models.transaction_invoice_replacement import TransactionInvoiceReplacement
from wallee.models.transaction_invoice_state import TransactionInvoiceState
from wallee.models.transaction_line_item_version import TransactionLineItemVersion
from wallee.models.transaction_line_item_version_create import TransactionLineItemVersionCreate
from wallee.models.transaction_line_item_version_state import TransactionLineItemVersionState
from wallee.models.transaction_list_response import TransactionListResponse
from wallee.models.transaction_pending import TransactionPending
from wallee.models.transaction_search_response import TransactionSearchResponse
from wallee.models.transaction_state import TransactionState
from wallee.models.transaction_user_interface_type import TransactionUserInterfaceType
from wallee.models.transaction_void import TransactionVoid
from wallee.models.transaction_void_list_response import TransactionVoidListResponse
from wallee.models.transaction_void_mode import TransactionVoidMode
from wallee.models.transaction_void_search_response import TransactionVoidSearchResponse
from wallee.models.transaction_void_state import TransactionVoidState
from wallee.models.two_factor_authentication_type import TwoFactorAuthenticationType
from wallee.models.user import User
from wallee.models.user_account_role import UserAccountRole
from wallee.models.user_account_role_list_response import UserAccountRoleListResponse
from wallee.models.user_space_role import UserSpaceRole
from wallee.models.user_space_role_list_response import UserSpaceRoleListResponse
from wallee.models.user_type import UserType
from wallee.models.wallet_type import WalletType
from wallee.models.web_app_confirmation_response import WebAppConfirmationResponse
from wallee.models.webhook_identity import WebhookIdentity
from wallee.models.webhook_listener import WebhookListener
from wallee.models.webhook_listener_create import WebhookListenerCreate
from wallee.models.webhook_listener_entity import WebhookListenerEntity
from wallee.models.webhook_listener_list_response import WebhookListenerListResponse
from wallee.models.webhook_listener_search_response import WebhookListenerSearchResponse
from wallee.models.webhook_listener_update import WebhookListenerUpdate
from wallee.models.webhook_url_list_response import WebhookURLListResponse
from wallee.models.webhook_url_search_response import WebhookURLSearchResponse
from wallee.models.webhook_url import WebhookUrl
from wallee.models.webhook_url_create import WebhookUrlCreate
from wallee.models.webhook_url_update import WebhookUrlUpdate



# coding: utf-8

"""
Wallee AG Python SDK

This library allows to interact with the Wallee AG payment service.

Copyright owner: Wallee AG
Website: https://en.wallee.com
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


import warnings
from pydantic import validate_call, Field, StrictFloat, StrictStr, StrictInt
from typing import Any, Dict, List, Optional, Tuple, Union
from typing_extensions import Annotated

from pydantic import Field, StrictStr
from typing import List, Optional
from typing_extensions import Annotated
from wallee.models.country_list_response import CountryListResponse
from wallee.models.country_search_response import CountrySearchResponse
from wallee.models.rest_country import RestCountry
from wallee.models.rest_country_state import RestCountryState

from wallee.api_client import ApiClient, RequestSerialized
from wallee.api_response import ApiResponse
from wallee.rest import RESTResponseType



class CountriesService:
    def __init__(self, configuration):
        self.api_client = ApiClient(configuration=configuration)


    @validate_call
    def get_countries(
        self,
        expand: Optional[List[StrictStr]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> CountryListResponse:
        """List all countries



        :param expand:
        :type expand: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_countries_serialize(
            expand=expand,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CountryListResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_countries_with_http_info(
        self,
        expand: Optional[List[StrictStr]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[CountryListResponse]:
        """List all countries



        :param expand:
        :type expand: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_countries_serialize(
            expand=expand,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CountryListResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_countries_without_preload_content(
        self,
        expand: Optional[List[StrictStr]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List all countries



        :param expand:
        :type expand: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_countries_serialize(
            expand=expand,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CountryListResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _get_countries_serialize(
        self,
        expand,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'expand': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if expand is not None:
            
            _query_params.append(('expand', expand))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='GET',
            resource_path='/countries',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )





    @validate_call
    def get_countries_code(
        self,
        code: StrictStr,
        expand: Optional[List[StrictStr]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RestCountry:
        """Retrieve a country



        :param code: (required)
        :type code: str
        :param expand:
        :type expand: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_countries_code_serialize(
            code=code,
            expand=expand,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RestCountry",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_countries_code_with_http_info(
        self,
        code: StrictStr,
        expand: Optional[List[StrictStr]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[RestCountry]:
        """Retrieve a country



        :param code: (required)
        :type code: str
        :param expand:
        :type expand: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_countries_code_serialize(
            code=code,
            expand=expand,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RestCountry",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_countries_code_without_preload_content(
        self,
        code: StrictStr,
        expand: Optional[List[StrictStr]] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Retrieve a country



        :param code: (required)
        :type code: str
        :param expand:
        :type expand: List[str]
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_countries_code_serialize(
            code=code,
            expand=expand,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RestCountry",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _get_countries_code_serialize(
        self,
        code,
        expand,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'expand': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if code is not None:
            _path_params['code'] = code
        # process the query parameters
        if expand is not None:
            
            _query_params.append(('expand', expand))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='GET',
            resource_path='/countries/{code}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )





    @validate_call
    def get_countries_country_code_states(
        self,
        country_code: StrictStr,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[RestCountryState]:
        """List all states for a country



        :param country_code: (required)
        :type country_code: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_countries_country_code_states_serialize(
            country_code=country_code,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[RestCountryState]",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_countries_country_code_states_with_http_info(
        self,
        country_code: StrictStr,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[RestCountryState]]:
        """List all states for a country



        :param country_code: (required)
        :type country_code: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_countries_country_code_states_serialize(
            country_code=country_code,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[RestCountryState]",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_countries_country_code_states_without_preload_content(
        self,
        country_code: StrictStr,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List all states for a country



        :param country_code: (required)
        :type country_code: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_countries_country_code_states_serialize(
            country_code=country_code,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[RestCountryState]",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _get_countries_country_code_states_serialize(
        self,
        country_code,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if country_code is not None:
            _path_params['countryCode'] = country_code
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='GET',
            resource_path='/countries/{countryCode}/states',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )





    @validate_call
    def get_countries_search(
        self,
        expand: Optional[List[StrictStr]] = None,
        limit: Annotated[Optional[Annotated[int, Field(le=100, strict=True, ge=1)]], Field(description="A limit on the number of objects to be returned, between 1 and 100. Default is 10.")] = None,
        offset: Annotated[Optional[Annotated[int, Field(le=10000, strict=True)]], Field(description="A cursor for pagination, specifies the number of objects to skip.")] = None,
        order: Annotated[Optional[StrictStr], Field(description="The fields and order to sort the objects by.")] = None,
        query: Annotated[Optional[StrictStr], Field(description="The search query to filter the objects by.")] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> CountrySearchResponse:
        """Search countries



        :param expand:
        :type expand: List[str]
        :param limit: A limit on the number of objects to be returned, between 1 and 100. Default is 10.
        :type limit: int
        :param offset: A cursor for pagination, specifies the number of objects to skip.
        :type offset: int
        :param order: The fields and order to sort the objects by.
        :type order: str
        :param query: The search query to filter the objects by.
        :type query: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_countries_search_serialize(
            expand=expand,
            limit=limit,
            offset=offset,
            order=order,
            query=query,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CountrySearchResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_countries_search_with_http_info(
        self,
        expand: Optional[List[StrictStr]] = None,
        limit: Annotated[Optional[Annotated[int, Field(le=100, strict=True, ge=1)]], Field(description="A limit on the number of objects to be returned, between 1 and 100. Default is 10.")] = None,
        offset: Annotated[Optional[Annotated[int, Field(le=10000, strict=True)]], Field(description="A cursor for pagination, specifies the number of objects to skip.")] = None,
        order: Annotated[Optional[StrictStr], Field(description="The fields and order to sort the objects by.")] = None,
        query: Annotated[Optional[StrictStr], Field(description="The search query to filter the objects by.")] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[CountrySearchResponse]:
        """Search countries



        :param expand:
        :type expand: List[str]
        :param limit: A limit on the number of objects to be returned, between 1 and 100. Default is 10.
        :type limit: int
        :param offset: A cursor for pagination, specifies the number of objects to skip.
        :type offset: int
        :param order: The fields and order to sort the objects by.
        :type order: str
        :param query: The search query to filter the objects by.
        :type query: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_countries_search_serialize(
            expand=expand,
            limit=limit,
            offset=offset,
            order=order,
            query=query,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CountrySearchResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_countries_search_without_preload_content(
        self,
        expand: Optional[List[StrictStr]] = None,
        limit: Annotated[Optional[Annotated[int, Field(le=100, strict=True, ge=1)]], Field(description="A limit on the number of objects to be returned, between 1 and 100. Default is 10.")] = None,
        offset: Annotated[Optional[Annotated[int, Field(le=10000, strict=True)]], Field(description="A cursor for pagination, specifies the number of objects to skip.")] = None,
        order: Annotated[Optional[StrictStr], Field(description="The fields and order to sort the objects by.")] = None,
        query: Annotated[Optional[StrictStr], Field(description="The search query to filter the objects by.")] = None,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Search countries



        :param expand:
        :type expand: List[str]
        :param limit: A limit on the number of objects to be returned, between 1 and 100. Default is 10.
        :type limit: int
        :param offset: A cursor for pagination, specifies the number of objects to skip.
        :type offset: int
        :param order: The fields and order to sort the objects by.
        :type order: str
        :param query: The search query to filter the objects by.
        :type query: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_countries_search_serialize(
            expand=expand,
            limit=limit,
            offset=offset,
            order=order,
            query=query,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "CountrySearchResponse",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _get_countries_search_serialize(
        self,
        expand,
        limit,
        offset,
        order,
        query,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
            'expand': 'multi',
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        if expand is not None:
            
            _query_params.append(('expand', expand))
            
        if limit is not None:
            
            _query_params.append(('limit', limit))
            
        if offset is not None:
            
            _query_params.append(('offset', offset))
            
        if order is not None:
            
            _query_params.append(('order', order))
            
        if query is not None:
            
            _query_params.append(('query', query))
            
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='GET',
            resource_path='/countries/search',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )





    @validate_call
    def get_countries_states(
        self,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> List[RestCountryState]:
        """List all country states



        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_countries_states_serialize(
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[RestCountryState]",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_countries_states_with_http_info(
        self,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[List[RestCountryState]]:
        """List all country states



        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_countries_states_serialize(
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[RestCountryState]",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_countries_states_without_preload_content(
        self,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """List all country states



        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_countries_states_serialize(
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "List[RestCountryState]",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _get_countries_states_serialize(
        self,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='GET',
            resource_path='/countries/states',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )





    @validate_call
    def get_countries_states_id(
        self,
        id: StrictStr,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RestCountryState:
        """Retrieve a country state



        :param id: (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_countries_states_id_serialize(
            id=id,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RestCountryState",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        ).data


    @validate_call
    def get_countries_states_id_with_http_info(
        self,
        id: StrictStr,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> ApiResponse[RestCountryState]:
        """Retrieve a country state



        :param id: (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_countries_states_id_serialize(
            id=id,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RestCountryState",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }

        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        response_data.read()
        return self.api_client.response_deserialize(
            response_data=response_data,
            response_types_map=_response_types_map,
        )


    @validate_call
    def get_countries_states_id_without_preload_content(
        self,
        id: StrictStr,
        _content_type: Optional[StrictStr] = None,
        _headers: Optional[Dict[StrictStr, Any]] = None,
        _host_index: Annotated[StrictInt, Field(ge=0, le=0)] = 0,
    ) -> RESTResponseType:
        """Retrieve a country state



        :param id: (required)
        :type id: str
        :param _request_timeout: timeout setting for this request. If one
                                 number provided, it will be total request
                                 timeout. It can also be a pair (tuple) of
                                 (connection, read) timeouts.
        :type _request_timeout: int, tuple(int, int), optional
        :param _content_type: force content-type for the request.
        :type _content_type: str, Optional
        :param _headers: set to override the headers for a single
                         request; this effectively ignores the headers
                         in the spec for a single request.
        :type _headers: dict, optional
        :param _host_index: set to override the host_index for a single
                            request; this effectively ignores the host_index
                            in the spec for a single request.
        :type _host_index: int, optional
        :return: Returns the result object.
        """ # noqa: E501

        _param = self._get_countries_states_id_serialize(
            id=id,
            _content_type=_content_type,
            _headers=_headers,
            _host_index=_host_index
        )

        _response_types_map: Dict[str, Optional[str]] = {
            '200': "RestCountryState",
            '400': "RestApiErrorResponse",
            '401': "RestApiErrorResponse",
            '403': "RestApiErrorResponse",
            '404': "RestApiErrorResponse",
            '406': "RestApiErrorResponse",
            '415': "RestApiErrorResponse",
            '422': "RestApiErrorResponse",
            '429': "RestApiErrorResponse",
            '5XX': "RestApiErrorResponse",
        }
        response_data = self.api_client.call_api(
            *_param,
            _request_timeout = self.api_client.configuration.request_timeout
        )
        return response_data.response


    def _get_countries_states_id_serialize(
        self,
        id,
        _content_type,
        _headers,
        _host_index,
    ) -> RequestSerialized:

        _host = None

        _collection_formats: Dict[str, str] = {
        }

        _path_params: Dict[str, str] = {}
        _query_params: List[Tuple[str, str]] = []
        _header_params: Dict[str, Optional[str]] = _headers or {}
        _form_params: List[Tuple[str, str]] = []
        _files: Dict[
            str, Union[str, bytes, List[str], List[bytes], List[Tuple[str, bytes]]]
        ] = {}
        _body_params: Optional[bytes] = None

        # process the path parameters
        if id is not None:
            _path_params['id'] = id
        # process the query parameters
        # process the header parameters
        # process the form parameters
        # process the body parameter


        # set the HTTP header `Accept`
        if 'Accept' not in _header_params:
            _header_params['Accept'] = self.api_client.select_header_accept(
                [
                    'application/json'
                ]
            )


        return self.api_client.param_serialize(
            method='GET',
            resource_path='/countries/states/{id}',
            path_params=_path_params,
            query_params=_query_params,
            header_params=_header_params,
            body=_body_params,
            post_params=_form_params,
            files=_files,
            collection_formats=_collection_formats,
            _host=_host
        )






# coding: utf-8

"""
Wallee AG Python SDK

This library allows to interact with the Wallee AG payment service.

Copyright owner: Wallee AG
Website: https://en.wallee.com
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictFloat, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional, Union
from wallee.models.subscription_product_component_group import SubscriptionProductComponentGroup
from wallee.models.subscription_product_component_reference import SubscriptionProductComponentReference
from wallee.models.tax_class import TaxClass
from typing import Optional, Set
from typing_extensions import Self

class SubscriptionProductComponent(BaseModel):
    """
    SubscriptionProductComponent
    """ # noqa: E501
    tax_class: Optional[TaxClass] = Field(default=None, alias="taxClass")
    description: Optional[Dict[str, StrictStr]] = Field(default=None, description="The localized description of the component that is displayed to the customer.")
    component_change_weight: Optional[StrictInt] = Field(default=None, description="If switching from a component with a lower tier to a component with a higher one, this is considered an upgrade and a fee may be applied.", alias="componentChangeWeight")
    maximal_quantity: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="A maximum of the defined quantity can be selected for this component.", alias="maximalQuantity")
    version: Optional[StrictInt] = Field(default=None, description="The version is used for optimistic locking and incremented whenever the object is updated.")
    minimal_quantity: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="A minimum of the defined quantity must be selected for this component.", alias="minimalQuantity")
    reference: Optional[SubscriptionProductComponentReference] = None
    linked_space_id: Optional[StrictInt] = Field(default=None, description="The ID of the space this object belongs to.", alias="linkedSpaceId")
    quantity_step: Optional[Union[StrictFloat, StrictInt]] = Field(default=None, description="The quantity step determines the interval in which the quantity can be increased.", alias="quantityStep")
    sort_order: Optional[StrictInt] = Field(default=None, description="When listing components, they can be sorted by this number.", alias="sortOrder")
    component_group: Optional[SubscriptionProductComponentGroup] = Field(default=None, alias="componentGroup")
    name: Optional[Dict[str, StrictStr]] = Field(default=None, description="The localized name of the component that is displayed to the customer.")
    id: Optional[StrictInt] = Field(default=None, description="A unique identifier for the object.")
    default_component: Optional[StrictBool] = Field(default=None, description="Whether this is the default component in its group and preselected.", alias="defaultComponent")
    __properties: ClassVar[List[str]] = ["taxClass", "description", "componentChangeWeight", "maximalQuantity", "version", "minimalQuantity", "reference", "linkedSpaceId", "quantityStep", "sortOrder", "componentGroup", "name", "id", "defaultComponent"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of SubscriptionProductComponent from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "description",
            "component_change_weight",
            "maximal_quantity",
            "version",
            "minimal_quantity",
            "linked_space_id",
            "quantity_step",
            "sort_order",
            "name",
            "id",
            "default_component",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of tax_class
        if self.tax_class:
            _dict['taxClass'] = self.tax_class.to_dict()
        # override the default output from pydantic by calling `to_dict()` of reference
        if self.reference:
            _dict['reference'] = self.reference.to_dict()
        # override the default output from pydantic by calling `to_dict()` of component_group
        if self.component_group:
            _dict['componentGroup'] = self.component_group.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of SubscriptionProductComponent from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "taxClass": TaxClass.from_dict(obj["taxClass"]) if obj.get("taxClass") is not None else None,
            "description": obj.get("description"),
            "componentChangeWeight": obj.get("componentChangeWeight"),
            "maximalQuantity": obj.get("maximalQuantity"),
            "version": obj.get("version"),
            "minimalQuantity": obj.get("minimalQuantity"),
            "reference": SubscriptionProductComponentReference.from_dict(obj["reference"]) if obj.get("reference") is not None else None,
            "linkedSpaceId": obj.get("linkedSpaceId"),
            "quantityStep": obj.get("quantityStep"),
            "sortOrder": obj.get("sortOrder"),
            "componentGroup": SubscriptionProductComponentGroup.from_dict(obj["componentGroup"]) if obj.get("componentGroup") is not None else None,
            "name": obj.get("name"),
            "id": obj.get("id"),
            "defaultComponent": obj.get("defaultComponent")
        })
        return _obj





# coding: utf-8

"""
Wallee AG Python SDK

This library allows to interact with the Wallee AG payment service.

Copyright owner: Wallee AG
Website: https://en.wallee.com
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from datetime import datetime
from pydantic import BaseModel, ConfigDict, Field, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from wallee.models.charge_state import ChargeState
from wallee.models.charge_type import ChargeType
from wallee.models.failure_reason import FailureReason
from wallee.models.transaction import Transaction
from typing import Optional, Set
from typing_extensions import Self

class Charge(BaseModel):
    """
    Charge
    """ # noqa: E501
    planned_purge_date: Optional[datetime] = Field(default=None, description="The date and time when the object is planned to be permanently removed. If the value is empty, the object will not be removed.", alias="plannedPurgeDate")
    time_zone: Optional[StrictStr] = Field(default=None, description="The time zone that this object is associated with.", alias="timeZone")
    language: Optional[StrictStr] = Field(default=None, description="The language that is linked to the object.")
    space_view_id: Optional[StrictInt] = Field(default=None, description="The ID of the space view this object is linked to.", alias="spaceViewId")
    type: Optional[ChargeType] = None
    user_failure_message: Optional[StrictStr] = Field(default=None, description="The message that can be displayed to the customer explaining why the charge failed, in the customer's language.", alias="userFailureMessage")
    created_on: Optional[datetime] = Field(default=None, description="The date and time when the object was created.", alias="createdOn")
    version: Optional[StrictInt] = Field(default=None, description="The version is used for optimistic locking and incremented whenever the object is updated.")
    linked_space_id: Optional[StrictInt] = Field(default=None, description="The ID of the space this object belongs to.", alias="linkedSpaceId")
    timeout_on: Optional[datetime] = Field(default=None, description="The date and time when the charge will expire.", alias="timeoutOn")
    failure_reason: Optional[FailureReason] = Field(default=None, alias="failureReason")
    id: Optional[StrictInt] = Field(default=None, description="A unique identifier for the object.")
    state: Optional[ChargeState] = None
    transaction: Optional[Transaction] = None
    __properties: ClassVar[List[str]] = ["plannedPurgeDate", "timeZone", "language", "spaceViewId", "type", "userFailureMessage", "createdOn", "version", "linkedSpaceId", "timeoutOn", "failureReason", "id", "state", "transaction"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of Charge from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "planned_purge_date",
            "time_zone",
            "language",
            "space_view_id",
            "user_failure_message",
            "created_on",
            "version",
            "linked_space_id",
            "timeout_on",
            "id",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of failure_reason
        if self.failure_reason:
            _dict['failureReason'] = self.failure_reason.to_dict()
        # override the default output from pydantic by calling `to_dict()` of transaction
        if self.transaction:
            _dict['transaction'] = self.transaction.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of Charge from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "plannedPurgeDate": obj.get("plannedPurgeDate"),
            "timeZone": obj.get("timeZone"),
            "language": obj.get("language"),
            "spaceViewId": obj.get("spaceViewId"),
            "type": obj.get("type"),
            "userFailureMessage": obj.get("userFailureMessage"),
            "createdOn": obj.get("createdOn"),
            "version": obj.get("version"),
            "linkedSpaceId": obj.get("linkedSpaceId"),
            "timeoutOn": obj.get("timeoutOn"),
            "failureReason": FailureReason.from_dict(obj["failureReason"]) if obj.get("failureReason") is not None else None,
            "id": obj.get("id"),
            "state": obj.get("state"),
            "transaction": Transaction.from_dict(obj["transaction"]) if obj.get("transaction") is not None else None
        })
        return _obj





# coding: utf-8

"""
Wallee AG Python SDK

This library allows to interact with the Wallee AG payment service.

Copyright owner: Wallee AG
Website: https://en.wallee.com
Developer email: ecosystem-team@wallee.com

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

     http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.
"""


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, StrictStr
from typing import Any, ClassVar, Dict, List, Optional
from wallee.models.customers_presence import CustomersPresence
from wallee.models.data_collection_type import DataCollectionType
from wallee.models.payment_connector_feature import PaymentConnectorFeature
from wallee.models.payment_method import PaymentMethod
from wallee.models.payment_method_brand import PaymentMethodBrand
from wallee.models.payment_primary_risk_taker import PaymentPrimaryRiskTaker
from wallee.models.payment_processor import PaymentProcessor
from typing import Optional, Set
from typing_extensions import Self

class PaymentConnector(BaseModel):
    """
    PaymentConnector
    """ # noqa: E501
    supported_features: Optional[List[PaymentConnectorFeature]] = Field(default=None, description="The features that are supported by the connector.", alias="supportedFeatures")
    supported_customers_presences: Optional[List[CustomersPresence]] = Field(default=None, description="The types of customer's presence that are supported by the connector.", alias="supportedCustomersPresences")
    data_collection_type: Optional[DataCollectionType] = Field(default=None, alias="dataCollectionType")
    deprecated: Optional[StrictBool] = Field(default=None, description="Whether the object was deprecated.")
    primary_risk_taker: Optional[PaymentPrimaryRiskTaker] = Field(default=None, alias="primaryRiskTaker")
    description: Optional[Dict[str, StrictStr]] = Field(default=None, description="The localized description of the object.")
    payment_method_brand: Optional[PaymentMethodBrand] = Field(default=None, alias="paymentMethodBrand")
    processor: Optional[PaymentProcessor] = None
    deprecation_reason: Optional[Dict[str, StrictStr]] = Field(default=None, description="The deprecation reason describes why the object was deprecated.", alias="deprecationReason")
    supported_currencies: Optional[List[StrictStr]] = Field(default=None, description="The currencies that are supported by the connector.", alias="supportedCurrencies")
    name: Optional[Dict[str, StrictStr]] = Field(default=None, description="The localized name of the object.")
    payment_method: Optional[PaymentMethod] = Field(default=None, alias="paymentMethod")
    id: Optional[StrictInt] = Field(default=None, description="A unique identifier for the object.")
    __properties: ClassVar[List[str]] = ["supportedFeatures", "supportedCustomersPresences", "dataCollectionType", "deprecated", "primaryRiskTaker", "description", "paymentMethodBrand", "processor", "deprecationReason", "supportedCurrencies", "name", "paymentMethod", "id"]

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of PaymentConnector from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        * OpenAPI `readOnly` fields are excluded.
        """
        excluded_fields: Set[str] = set([
            "supported_features",
            "supported_customers_presences",
            "deprecated",
            "description",
            "deprecation_reason",
            "supported_currencies",
            "name",
            "id",
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        # override the default output from pydantic by calling `to_dict()` of each item in supported_features (list)
        _items = []
        if self.supported_features:
            for _item_supported_features in self.supported_features:
                if _item_supported_features:
                    _items.append(_item_supported_features.to_dict())
            _dict['supportedFeatures'] = _items
        # override the default output from pydantic by calling `to_dict()` of payment_method_brand
        if self.payment_method_brand:
            _dict['paymentMethodBrand'] = self.payment_method_brand.to_dict()
        # override the default output from pydantic by calling `to_dict()` of processor
        if self.processor:
            _dict['processor'] = self.processor.to_dict()
        # override the default output from pydantic by calling `to_dict()` of payment_method
        if self.payment_method:
            _dict['paymentMethod'] = self.payment_method.to_dict()
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of PaymentConnector from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "supportedFeatures": [PaymentConnectorFeature.from_dict(_item) for _item in obj["supportedFeatures"]] if obj.get("supportedFeatures") is not None else None,
            "supportedCustomersPresences": obj.get("supportedCustomersPresences"),
            "dataCollectionType": obj.get("dataCollectionType"),
            "deprecated": obj.get("deprecated"),
            "primaryRiskTaker": obj.get("primaryRiskTaker"),
            "description": obj.get("description"),
            "paymentMethodBrand": PaymentMethodBrand.from_dict(obj["paymentMethodBrand"]) if obj.get("paymentMethodBrand") is not None else None,
            "processor": PaymentProcessor.from_dict(obj["processor"]) if obj.get("processor") is not None else None,
            "deprecationReason": obj.get("deprecationReason"),
            "supportedCurrencies": obj.get("supportedCurrencies"),
            "name": obj.get("name"),
            "paymentMethod": PaymentMethod.from_dict(obj["paymentMethod"]) if obj.get("paymentMethod") is not None else None,
            "id": obj.get("id")
        })
        return _obj


